/*
  $VER: CfgGetNET.thor 1.5 (15.11.96)
  by Remco van Hooff

  Configuration editor for GetNET 2.2

  HISTORY
  1.0 - First release
  1.1 - Added GUI-FTP hotlist.
      - Added version handler.
      - Removed loading of rexxsupport.library, wasn't used :.)
  1.2 - Fixed a bug of requester not showing when configfile was missing.
  1.3 - Added AmFTP hotlist.
      - Added FTPSiteName variable.
      - Cleaned up the code a bit.
      - Changed the hotlist listview layout.
  1.4 - Added default values for missing options or hotlist entries, no need
        to update the config file with the installer anymore. Just run
        CfgGetNET.thor, edit the config file (no changes will be made to the
        existing one), save it and the new options are added.
      - Added tcpdl hotlist.
      - Added TCP/IP startup command for AmFTP, with some code from
        CfgSortMail.thor by Eirik Nicolai Synnes (AddAsterix).
  1.5 - Cancelling the hotlist prefs window didn't return a correct result.
      - No more leftovers (Accept and return) from a previous hotlist prefs.
      - Changes the way the TCPdl pre- and suffix are stored in the config
        file.
      - Used the wrong variable to display the TCPdl hotlist.
      - Forgot to parse on the TCPdl prefs, they are now saved properly.

  TODO
  - Rewrite the format of the config file to allow easy expandability.
*/

/*!~ "Variables" */
version = SUBWORD(SOURCELINE(2), 3,1)
cfgpath = 'env:thor/'
cfgfile = 'GetNET.config'
change = 0
title = 'CfgGetNET' version ' by Remco van Hooff'
/*~!*/

/*!~ "Init" */
OPTIONS RESULTS
/*TRACE RESULTS*/

p=' '||ADDRESS()||' '||SHOW('P',,)
IF POS(' THOR.',p)>0 THEN thorport=WORD(SUBSTR(p,POS(' THOR.',p)+1),1)
ELSE DO
  SAY 'THOR port not found!'
  EXIT 10
END

IF ~SHOW('p', 'BBSREAD') THEN DO
  ADDRESS COMMAND
    "run >nil: `GetEnv THOR/THORPath`bin/LoadBBSRead"
    "WaitForPort BBSREAD"
END
/*~!*/

CALL loadprefs
CALL cfgmain

EXIT

/*!~ "LoadPrefs" */
loadprefs:
  IF ~EXISTS(cfgpath||cfgfile) THEN DO
    Address(thorport)
    'Requestnotify TEXT "Could not find the configuration file.\nUsing the default settings." BT "_OK"'
    Call checkprefs
  END
  ELSE DO
    CALL OPEN(prf,cfgpath||cfgfile,'R')
      DO UNTIL EOF(prf)
        line = READLN(prf)
        SELECT
          WHEN UPPER(WORD(line,1)) = 'BBS' THEN DO
            bbs = SUBWORD(line,2)
          END
          WHEN UPPER(WORD(line,1)) = 'ASKFR' THEN DO
            askfr = WORD(line,2)
          END
          WHEN Upper(Word(line,1)) = 'REQDIR' THEN DO
            reqdir = SubWord(line,2)
          END
          WHEN Upper(Word(line,1)) = 'FTPSAVEMODE' THEN DO
            ftpsavemode = UPPER(WORD(line,2))
          END
          WHEN Upper(Word(line,1)) = 'FTPSITENAME' THEN DO
            ftpsitename = UPPER(WORD(line,2))
          END
          WHEN UPPER(WORD(line,1)) = 'AMOSAIC' THEN DO
            hotlist_amosaic = WORD(line,2)
            amosaic = WORD(line,3)
          END
          WHEN UPPER(WORD(line,1)) = 'AWEB' THEN DO
            hotlist_aweb = WORD(line,2)
            aweb = WORD(line,3)
          END
          WHEN UPPER(WORD(line,1)) = 'IBROWSE' THEN DO
            hotlist_ibrowse = WORD(line,2)
            ibrowse = WORD(line,3)
          END
          WHEN UPPER(WORD(line,1)) = 'VOYAGER' THEN DO
            hotlist_voyager = WORD(line,2)
            voyager = WORD(line,3)
          END
          WHEN UPPER(WORD(line,1)) = 'HTML' THEN DO
            hotlist_html = WORD(line,2)
            html = WORD(line,3)
          END
          WHEN UPPER(WORD(line,1)) = 'NCFTP' THEN DO
            hotlist_ncftp = WORD(line,2)
            ncftp = WORD(line,3)
          END
          WHEN UPPER(WORD(line,1)) = 'DOPUS' THEN DO
            hotlist_dopus = WORD(line,2)
            dopus = WORD(line,3)
          END
          WHEN UPPER(WORD(line,1)) = 'GUIFTP' THEN DO
            hotlist_guiftp = WORD(line,2)
            guiftp = WORD(line,3)
          END
          WHEN UPPER(WORD(line,1)) = 'AMFTP' THEN DO
            path_amftp = WORD(line,2)
            amftp = WORD(line,3)
          END
          WHEN UPPER(WORD(line,1)) = 'TCPDL' THEN DO
            hotlist_tcpdl = WORD(line,2)
            tcpdl = WORD(line,3)
          END
          WHEN UPPER(WORD(line,1)) = 'TCPDL_PRE' THEN DO
            pre_tcpdl = SUBWORD(line,2)
          END
          WHEN UPPER(WORD(line,1)) = 'TCPDL_SUF' THEN DO
            suf_tcpdl = SUBWORD(line,2)
          END
          WHEN UPPER(WORD(line,1)) = 'TCP-IP' THEN DO
            tcp_ip = SUBWORD(line,2)
          END
          OTHERWISE NOP
        END
      END
    CALL CLOSE(prf)
    CALL checkprefs
  END
RETURN
/*~!*/

/*!~ "CheckPrefs" */
checkprefs:
  IF (SYMBOL('bbs') ~= 'VAR' | bbs = '') THEN bbs = 'Internet'
  IF ~DATATYPE(askfr, 'BIN') THEN askfr = 0
  IF (SYMBOL('reqdir') ~= 'VAR' | reqdir = '') THEN reqdir = 'RAM:'
  IF ~(ftpsavemode = 'B' | ftpsavemode = 'W' | ftpsavemode = 'F') THEN ftpsavemode = 'B'
  IF ~DATATYPE(ftpsitename, 'BIN') THEN ftpsitename = 0

  IF ~DATATYPE(amosaic, 'BIN') THEN amosaic = 0
  IF (SYMBOL('hotlist_amosaic') ~= 'VAR' | hotlist_amosaic = '') THEN hotlist_amosaic = 'envarc:mosaic/.mosaic-hotlist-default'

  IF ~DATATYPE(aweb, 'BIN') THEN aweb = 0
  IF (SYMBOL('hotlist_aweb') ~= 'VAR' | hotlist_aweb = '') THEN hotlist_aweb = 'aweb:aweb.hotlist'

  IF ~DATATYPE(ibrowse, 'BIN') THEN ibrowse = 0
  IF (SYMBOL('hotlist_ibrowse') ~= 'VAR' | hotlist_ibrowse = '') THEN hotlist_ibrowse = 'ibrowse:ibrowse-hotlist.html'

  IF ~DATATYPE(voyager, 'BIN') THEN voyager = 0
  IF (SYMBOL('hotlist_voyager') ~= 'VAR' | hotlist_voyager = '') THEN hotlist_voyager = 'voyager:bookmarks.html'

  IF ~DATATYPE(html, 'BIN') THEN html = 0
  IF (SYMBOL('hotlist_html') ~= 'VAR' | hotlist_html = '') THEN hotlist_html = 'sys:hotlist.html'

  IF ~DATATYPE(ncftp, 'BIN') THEN ncftp = 0
  IF (SYMBOL('hotlist_ncftp') ~= 'VAR' | hotlist_ncftp = '') THEN hotlist_ncftp = 'home:.ncrecent'

  IF ~DATATYPE(dopus, 'BIN') THEN dopus = 0
  IF (SYMBOL('hotlist_dopus') ~= 'VAR' | hotlist_dopus = '') THEN hotlist_dopus = 'dopus5:system/ftp.config'

  IF ~DATATYPE(guiftp, 'BIN') THEN guiftp = 0
  IF (SYMBOL('hotlist_guiftp') ~= 'VAR' | hotlist_guiftp = '') THEN hotlist_guiftp = 'AmiTCP:FTP/Gui-FTP.rc'

  IF ~DATATYPE(amftp, 'BIN') THEN amftp = 0
  IF (SYMBOL('path_amftp') ~= 'VAR' | hotlist_amftp = '') THEN path_amftp = 'AmiTCP:FTP/AmFTP'

  IF ~DATATYPE(tcpdl, 'BIN') THEN tcpdl = 0
  IF (SYMBOL(hotlist_tcpdl) ~= 'VAR' | hotlist_tcpdl = '') THEN hotlist_tcpdl = 'tcpdldir:urllist'
  IF SYMBOL('pre_tcpdl') ~= 'VAR' THEN pre_tcpdl = ''
  IF SYMBOL('suf_tcpdl') ~= 'VAR' THEN suf_tcpdl = ''

  IF (SYMBOL('tcp_ip') ~= 'VAR' | tcp_ip = '') THEN tcp_ip = 'RequestChoice >NIL: TITLE="GetNET ERROR" BODY="No TCP/IP stack to run. Use CfgGetNET to edit." GADGETS="OK" Pubscreen=Thor.1'
RETURN
/*~!*/

/*!~ "SavePrefs" */
saveprefs:
  CALL OPEN(prf,cfgpath||cfgfile,'W')
    CALL WRITELN(prf,'BBS' bbs)
    CALL WRITELN(prf,'ASKFR' askfr)
    CALL WRITELN(prf,'REQDIR' reqdir)
    CALL WRITELN(prf,'FTPSAVEMODE' ftpsavemode)
    CALL WRITELN(prf,'FTPSITENAME' ftpsitename)
    CALL WRITELN(prf,'AMOSAIC' hotlist_amosaic amosaic)
    CALL WRITELN(prf,'AWEB' hotlist_aweb aweb)
    CALL WRITELN(prf,'IBROWSE' hotlist_ibrowse ibrowse)
    CALL WRITELN(prf,'VOYAGER' hotlist_voyager voyager)
    CALL WRITELN(prf,'HTML' hotlist_html html)
    CALL WRITELN(prf,'NCFTP' hotlist_ncftp ncftp)
    CALL WRITELN(prf,'DOPUS' hotlist_dopus dopus)
    CALL WRITELN(prf,'GUIFTP' hotlist_guiftp guiftp)
    CALL WRITELN(prf,'AMFTP' path_amftp amftp)
    CALL WRITELN(prf,'TCPDL' hotlist_tcpdl tcpdl)
    CALL WRITELN(prf,'TCPDL_PRE' pre_tcpdl)
    CALL WRITELN(prf,'TCPDL_SUF' suf_tcpdl)
    CALL WRITELN(prf,'TCP-IP' tcp_ip)
  CALL CLOSE(prf)  
  ADDRESS command 'Copy "'cfgpath||cfgfile'" "ENVARC:Thor/" QUIET'
RETURN

/*~!*/

/*!~ "CfgMain" */
cfgmain:
  SELECT
    WHEN askfr = 0 THEN askfryn = 'No'
    WHEN askfr = 1 THEN askfryn = 'Yes'
    OTHERWISE askfryn = '?'
  END

  SELECT
    WHEN ftpsavemode = 'W' THEN fsm = 'WWW hotlist(s)'
    WHEN ftpsavemode = 'F' THEN fsm = 'FTP hotlist(s)'
    WHEN ftpsavemode = 'B' THEN fsm = 'FTP and WWW hotlists'
    OTHERWISE fsm = '?'
  END

  SELECT
    WHEN ((ftpsavemode = 'F' | ftpsavemode = 'B') & ftpsitename = 1) THEN fsn = 'URL'
    WHEN ((ftpsavemode = 'F' | ftpsavemode = 'B') & ftpsitename = 0) THEN fsn = 'Subject/filename'
    OTHERWISE fsn = '?'
  END

  main.1 = 'Internet system         : ' bbs
  main.2 = 'Popup File/Msg requester: ' askfryn
  main.3 = 'File requester dir      : ' reqdir
  main.4 = 'FTP save mode           : ' fsm
  main.5 = ''
  main.6 = 'Edit hotlists'
  main.7 = ''
  main.8 = 'Save and exit'
  main.count = 8

  IF (ftpsavemode = 'F' | ftpsavemode = 'B') THEN DO
    main.5 = 'FTP site name           : ' fsn
    main.6 = ''
    main.7 = 'Edit hotlists'
    main.8 = ''
    main.9 = 'Save and exit'
    main.count = 9
  END

  ADDRESS(thorport)
  'REQUESTLIST INSTEM' main 'TITLE "'title'" SIZEGADGET'
  IF (RC = 30) THEN DO
    'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
    EXIT
  END
  IF RC = 5 THEN DO
    IF change THEN DO
      'REQUESTNOTIFY TEXT "Are you sure you want to\nexit without saving?" BT "_Yes|_No"'
      IF RC = 0 THEN DO
        IF RESULT = 1 THEN EXIT
        IF RESULT = 0 THEN SIGNAL cfgmain
      END
    END
  END
  ELSE DO
    selected = RESULT
    SELECT
      WHEN POS('Internet system',selected) ~= 0 THEN DO
        CALL cfgsystem
        SIGNAL cfgmain
      END
      WHEN POS('Popup', selected) ~= 0 THEN DO
        askfr = askYN('Popup a requester asking to scan\nmessages or a file on disk.\n\nCurrently:' askfryn, askfryn, '_Yes|_No')
        SIGNAL cfgmain
      END
      WHEN Pos('File req', selected) ~= 0 THEN DO
        CALL cfgreqdir
        SIGNAL cfgmain
      END
      WHEN POS('FTP save', selected) ~= 0 THEN DO
        CALL cfgftpsavemode
        SIGNAL cfgmain
      END
      WHEN POS('FTP site', selected) ~= 0 THEN DO
        ftpsitename = askYN('Use the URL as FTP site description\nor the default subject/filename.\n\nCurrently:' fsn, fsn, '_URL|_Default')
        SIGNAL cfgmain
      END
      WHEN selected = 'Edit hotlists' THEN DO
        CALL cfghotlists
        SIGNAL cfgmain
      END
      WHEN selected = 'Save and exit' THEN DO
        CALL saveprefs
        EXIT
      END
      OTHERWISE CALL cfgmain
    END
  END
RETURN
/*~!*/

/*!~ "CfgSystem" */
cfgsystem:
  ADDRESS BBSREAD 'GETBBSLIST STEM' bbslst
  IF (RC = 30) THEN DO
    'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
    EXIT
  END
  ELSE DO
    'REQUESTLIST INSTEM' bbslst 'TITLE "Select your internet system." SIZEGADGET'
    IF RC = 30 THEN DO
      'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
      EXIT
    END
    IF RC = 5 THEN RETURN
    IF RC = 0 THEN DO
      bbs = RESULT
      change = 1
    END
  END
RETURN
/*~!*/

/*!~ "AskYN" */
askYN:
 reqtxt = ARG(1)
 option = ARG(2)
 buttons = ARG(3)
 'REQUESTNOTIFY TEXT "'reqtxt'" BT "'buttons'"'
  IF RC = 0 THEN DO
    IF RESULT = 1 THEN DO
      IF option = 0 THEN change = 1
      option = 1
    END
    IF RESULT = 0 THEN DO
      IF option = 1 THEN change = 1
      option = 0
    END
  END
RETURN(option)
/*~!*/

/*!~ "CfgReqDir" */
CfgReqDir:
  IF reqdir = '' THEN reqdir = 'ram:'
  'REQUESTFILE TITLE "Select the default dir" ID "'reqdir'" FP'
  IF (RC = 30) THEN DO
    'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
    EXIT
  END
  IF RC = 5 THEN RETURN
  IF RC = 0 THEN DO
    reqdir2 = reqdir
    reqdir = RESULT
    IF reqdir2 ~= reqdir THEN change = 1
  END
RETURN
/*~!*/

/*!~ "CfgFTPSaveMode" */
cfgftpsavemode:
  'REQUESTNOTIFY TEXT "Where to should the FTP addresses be saved?\n- WWW hotlist(s) only\n- FTP hotlist(s) only\n- WWW and FTP hotlists\n\nCurrently:' fsm'" BT "_WWW|_FTP|_Both"'
  IF (RC = 30) THEN DO
    'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
    EXIT
  END
  IF RC = 0 THEN DO
    IF RESULT = 1 THEN DO
      IF ftpsavemode ~= 'W' THEN change = 1
      ftpsavemode = 'W'
    END
    IF RESULT = 2 THEN DO
      IF ftpsavemode ~= 'F' THEN change = 1
      ftpsavemode = 'F'
    END
    IF RESULT = 0 THEN DO
      IF ftpsavemode ~= 'B' THEN change = 1
      ftpsavemode = 'B'
    END
  END
RETURN
/*~!*/

/*!~ "CfgHotlists" */
cfghotlists:
  DROP hot.
  hot.1  = '     -= WWW hotlists =-'
  hot.2  = 'AMosaic'
  hot.3  = 'AWeb'
  hot.4  = 'IBrowse'
  hot.5  = 'TCPdl'
  hot.6  = 'Voyager'
  hot.7  = 'HTML'
  hot.8  = ''
  hot.9  = 'Return'
  hot.count = 9

  IF (ftpsavemode = 'F' | ftpsavemode = 'B') THEN DO
    hot.9  = '     -= FTP hotlists =-'
    hot.10 = 'AmFTP'
    hot.11 = 'DOpusFTP'
    hot.12 = 'GUI-FTP'
    hot.13 = 'ncFTPrecent'
    hot.14 = ''
    hot.15 = 'Return'
    hot.count = 15
  END

  ADDRESS(thorport)
  'REQUESTLIST INSTEM' hot 'TITLE "Select hotlist to edit." SIZEGADGET'
  IF (RC = 30) THEN DO
    'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
    EXIT
  END
  IF RC = 5 THEN RETURN
  ELSE DO
    hotlist = RESULT
    SELECT
      WHEN UPPER(hotlist) = 'AMOSAIC' THEN DO
        CALL edithotlist(hotlist_amosaic, amosaic, hotlist)
        amosaic = substr(savedir,1,1)
        hotlist_amosaic = substr(savedir,2)
        SIGNAL cfghotlists
      END
      WHEN UPPER(hotlist) = 'AWEB' THEN DO
        CALL edithotlist(hotlist_aweb, aweb, hotlist)
        aweb = substr(savedir,1,1)
        hotlist_aweb = substr(savedir,2)
        SIGNAL cfghotlists
      END
      WHEN UPPER(hotlist) = 'IBROWSE' THEN DO
        CALL edithotlist(hotlist_ibrowse, ibrowse, hotlist)
        ibrowse = substr(savedir,1,1)
        hotlist_ibrowse = substr(savedir,2)
        SIGNAL cfghotlists
      END
      WHEN UPPER(hotlist) = 'VOYAGER' THEN DO
        CALL edithotlist(hotlist_voyager, voyager, hotlist)
        voyager = substr(savedir,1,1)
        hotlist_voyager = substr(savedir,2)
        SIGNAL cfghotlists
      END
      WHEN UPPER(hotlist) = 'HTML' THEN DO
        CALL edithotlist(hotlist_html, html, hotlist)
        html = substr(savedir,1,1)
        hotlist_html = substr(savedir,2)
        SIGNAL cfghotlists
      END
      WHEN UPPER(hotlist) = 'DOPUSFTP' THEN DO
        CALL edithotlist(hotlist_dopus, dopus, hotlist)
        dopus = substr(savedir,1,1)
        hotlist_dopus = substr(savedir,2)
        SIGNAL cfghotlists
      END
      WHEN UPPER(hotlist) = 'NCFTPRECENT' THEN DO
        CALL edithotlist(hotlist_ncftp, ncftp, hotlist)
        ncftp = substr(savedir,1,1)
        hotlist_ncftp = substr(savedir,2)
        SIGNAL cfghotlists
      END
      WHEN UPPER(hotlist) = 'GUI-FTP' THEN DO
        CALL edithotlist(hotlist_guiftp, guiftp, hotlist)
        guiftp = substr(savedir,1,1)
        hotlist_guiftp = substr(savedir,2)
        SIGNAL cfghotlists
      END
      WHEN UPPER(hotlist) = 'AMFTP' THEN DO
        CALL editamftp(path_amftp, amftp, tcp_ip)
        amftp = substr(savedir,1,1)
        path_amftp = substr(savedir,2)
        SIGNAL cfghotlists
      END
      WHEN UPPER(hotlist) = 'TCPDL' THEN DO
        CALL edittcpdl(hotlist_tcpdl, tcpdl, pre_tcpdl, suf_tcpdl)
        tcpdl = substr(savedir,1,1)
        hotlist_tcpdl = substr(savedir,2)
        SIGNAL cfghotlists
      END
      WHEN UPPER(hotlist) = 'RETURN' THEN RETURN
      OTHERWISE CALL cfghotlists
    END
  END
RETURN
/*~!*/

/*!~ "EditHotlist" */
edithotlist:
  dir  = ARG(1)
  save = ARG(2)
  name = ARG(3)

  SELECT
    WHEN save = 0 THEN save2 = 'No'
    WHEN save = 1 THEN save2 = 'Yes'
    OTHERWISE save2 = '?'
  END

  DROP edit.
  edit.1 = 'Path: ' dir
  edit.2 = 'Save: ' save2
  edit.3 = ''
  edit.4 = 'Accept and return'
  edit.count = 4

  ADDRESS(thorport)
  'REQUESTLIST INSTEM' edit 'TITLE "Edit the' name 'hotlist." SIZEGADGET'
  IF (RC = 30) THEN DO
    'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
    EXIT
  END
  IF RC = 5 THEN DO
    savedir = save||dir
    RETURN
  END
  ELSE DO
    option = RESULT
    SELECT
      WHEN POS('PATH', UPPER(option)) ~= 0 THEN DO
       CALL request_file
       CALL edithotlist(dir, save, name)
      END
      WHEN POS('SAVE', UPPER(option)) ~= 0 THEN DO
        save = askYN('Save addresses to the' name 'hotlist?\n\nCurrently:' save2, save, '_Yes|_No')
        CALL edithotlist(dir, save, name)
      END
      WHEN POS('ACCEPT', UPPER(option)) ~= 0 THEN DO
        savedir = save||dir /* there should be a better solution for this */
        RETURN
      END
      OTHERWISE CALL edithotlist(dir, save, name)
    END
  END
RETURN
/*~!*/

/*!~ "Request_File" */
request_file:
  filepos = LastPos('/', dir)
  IF filepos = 0 THEN filepos = LastPos(':', dir)
  file = substr(dir,filepos+1)
  filedir = substr(dir,1,filepos)

  'REQUESTFILE TITLE "Select the' name 'hotlist file." ID "'filedir'" IF "'file'" FP'
  IF (RC = 30) THEN DO
    'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
    EXIT
  END
  IF RC = 5 THEN RETURN
  IF RC = 0 THEN DO
    dir2 = dir
    dir = RESULT
    IF dir2 ~= dir THEN change = 1
  END
RETURN
/*~!*/

/*!~ "EditTCPdl" */
edittcpdl:
  dir  = ARG(1)
  save = ARG(2)
  pre  = ARG(3)
  suf  = ARG(4)

  SELECT
    WHEN save = 0 THEN save2 = 'No'
    WHEN save = 1 THEN save2 = 'Yes'
    OTHERWISE save2 = '?'
  END

  DROP edit.
  edit.1 = 'Path:   ' dir
  edit.2 = 'Save:   ' save2
  edit.3 = 'Prefix: ' pre
  edit.4 = 'Suffix: ' suf
  edit.5 = ''
  edit.6 = 'Accept and return'
  edit.count = 6

  ADDRESS(thorport)
  'REQUESTLIST INSTEM' edit 'TITLE "Edit the TCPdl hotlist." SIZEGADGET'
  IF (RC = 30) THEN DO
    'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
    EXIT
  END
  IF RC = 5 THEN DO
    savedir = save||dir
    RETURN
  END
  ELSE DO
    option = RESULT
    SELECT
      WHEN POS('PATH', UPPER(option)) ~= 0 THEN DO
        CALL request_file
        CALL edittcpdl(dir, save, pre, suf)
      END
      WHEN POS('SAVE', UPPER(option)) ~= 0 THEN DO
        save = askYN('Save addresses to the TCPdl hotlist?\n\nCurrently:' save2, save, '_Yes|_No')
        CALL edittcpdl(dir, save, pre, suf)
      END
      WHEN POS('PREFIX', UPPER(option)) ~= 0 THEN DO
        'REQUESTSTRING TITLE "'title'" BODY "Enter a prefix for the TCPdl hotlist entry" BT "_OK|Clea_r|_Cancel" ID "'pre'"'
        IF (RC = 30) THEN DO
          'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
          EXIT
        END
        IF RC = 5 THEN CALL edittcpdl(dir, save, pre_tcpdl, suf_tcpdl)
        IF RC = 0 THEN DO
          IF THORRC = 1 then pre_tcpdl = RESULT
          IF THORRC = 2 then pre_tcpdl = ''
          CALL edittcpdl(dir, save,pre_tcpdl,suf_tcpdl)
        END
      END
      WHEN POS('SUFFIX', UPPER(option)) ~= 0 THEN DO
        'REQUESTSTRING TITLE "'title'" BODY "Enter a suffix for the TCPdl hotlist entry" BT "_OK|Clea_r|_Cancel" ID "'suf'"'
        IF (RC = 30) THEN DO
          'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
          EXIT
        END
        IF RC = 5 THEN CALL edittcpdl(dir, save, pre_tcpdl, suf_tcpdl)
        IF RC = 0 THEN DO
          IF THORRC = 1 then suf_tcpdl = RESULT
          IF THORRC = 2 then suf_tcpdl = ''
          CALL edittcpdl(dir, save,pre_tcpdl,suf_tcpdl)
        END
      END
      WHEN POS('ACCEPT', UPPER(option)) ~= 0 THEN DO
        savedir = save||dir
        RETURN
      END
      OTHERWISE CALL edittcpdl(dir, save,pre_tcpdl,suf_tcpdl)
    END
  END
RETURN
/*~!*/

/*!~ "EditAmFTP" */
editamftp:
  dir  = ARG(1)
  save = ARG(2)
  tcp  = ARG(3)

  SELECT
    WHEN save = 0 THEN save2 = 'No'
    WHEN save = 1 THEN save2 = 'Yes'
    OTHERWISE save2 = '?'
  END

  DROP edit.
  edit.1 = 'Path:   ' path_amftp
  edit.2 = 'Save:   ' save2
  edit.3 = 'TCP/IP: ' LEFT(tcp,30)
  edit.5 = ''
  edit.6 = 'Accept and return'
  edit.count = 6

  ADDRESS(thorport)
  'REQUESTLIST INSTEM' edit 'TITLE "Edit the AmFTP hotlist." SIZEGADGET'
  IF (RC = 30) THEN DO
    'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
    EXIT
  END
  IF RC = 5 THEN DO
    savedir = save||dir
    RETURN
  END
  ELSE DO
    option = RESULT
    SELECT
      WHEN POS('PATH', UPPER(option)) ~= 0 THEN DO
       CALL request_file
       CALL editamftp(dir, save, tcp)
      END
      WHEN POS('SAVE', UPPER(option)) ~= 0 THEN DO
        save = askYN('Save addresses to the AmFTP hotlist?\n\nCurrently:' save2, save, '_Yes|_No')
        CALL editamftp(dir, save, tcp)
      END
      WHEN POS('TCP/IP', UPPER(option)) ~= 0 THEN DO
        'REQUESTSTRING TITLE "'title'" BODY "Enter a command to start your\nTCP-IP stack without going online." BT "_OK|_Cancel" MAXCHARS 256 ID "'addasterix(tcp)'"'
        IF (RC = 30) THEN DO
          'REQUESTNOTIFY TEXT "'THOR.LASTERROR'" BT "_Ok"'
          EXIT
        END
        IF RC = 5 THEN CALL editamftp(dir, save, tcp)
        IF RC = 0 THEN DO
          IF THORRC = 1 then tcp_ip = RESULT
          CALL editamftp(dir, save, tcp_ip)
        END
      END
      WHEN POS('ACCEPT', UPPER(option)) ~= 0 THEN DO
        savedir = save||dir /* there should be a better solution for this */
        RETURN
      END
      OTHERWISE CALL editamftp(dir, save,tcp)
    END
  END
RETURN
/*~!*/

/*!~ "AddAsterix" */
/* From CfgSortMail.thor by Eirik Nicolai Synnes */
addasterix:
  str = ARG(1)

  IF str = '' THEN RETURN(str)

  lastfound = 0; found = INDEX(str, '*')
  DO WHILE found > lastfound
    secondpart = SUBSTR(str, found + LENGTH('*'))
    firstpart = SUBSTR(str, 1, LENGTH(str) - LENGTH(SUBSTR(str, found)))
    str = firstpart || '**' || secondpart
    lastfound = found + LENGTH('**')
    found = INDEX(str, '*', lastfound)
  END

  lastfound = 0; found = INDEX(str, '"')
  DO WHILE found > lastfound
    secondpart = SUBSTR(str, found + LENGTH('"'))
    firstpart = SUBSTR(str, 1, LENGTH(str) - LENGTH(SUBSTR(str, found)))
    str = firstpart || '*"' || secondpart
    lastfound = found + LENGTH('*"')
    found = INDEX(str, '"', lastfound)
  END
RETURN(str)
/*~!*/
